require_relative 'flow_reference_v0_client'
require_relative 'version'
require_relative 'generated/currencies'
require_relative 'generated/countries'
require_relative 'generated/continents'
require_relative 'generated/languages'
require_relative 'generated/payment_methods'

module FlowCommerce
  module Reference
    class Currencies
      class << self
        def find(currency_code)
          code = currency_code.to_s.downcase.gsub(/[^\w]/, '').capitalize
          return nil unless code.length == 3
          return nil unless Data.respond_to?(code)
          Data.send(code)
        end

        def find!(currency_code)
          find(currency_code) || raise(ArgumentError, 'Currency "%s" is not found' % currency_code)
        end
      end
    end

    class Countries
      class << self
        def find(country_code)
          code = country_code.to_s.downcase.gsub(/[^\w]/,"").capitalize
          return nil unless [2,3].include?(code.length)
          return nil unless Data.respond_to?(code)
          Data.send(code)
        end

        def find!(country_code)
          find(country_code) || raise(ArgumentError, 'Country "%s" is not found' % country_code)
        end
      end
    end

    class Locales
      class << self
        def find(locale_code)
          code = locale_code.to_s.downcase.gsub(/[^\w]/,"").capitalize
          return nil unless code.length == 3
          return nil unless Data.respond_to?(code)
          Data.send(code)
        end

        def find!(locale_code)
          find(locale_code) || raise(ArgumentError, 'Country "%s" is not found' % locale_code)
        end
      end
    end

    class Continents
      class << self
        def find(continent_code)
          code = continent_code.to_s.downcase.gsub(/[^\w]/,"").capitalize
          return nil unless code.length == 3
          return nil unless Data.respond_to?(code)
          Data.send(code)
        end

        def find!(continent_code)
           find(continent_code) || raise(ArgumentError, 'Continent "%s" is not found' % continent_code)
        end
      end
    end

    class Languages
      class << self
        def find(language_code)
          code = language_code.to_s.downcase.gsub(/[^\w]/,"").capitalize
          return nil unless code.length == 2
          return nil unless Data.respond_to?(code)
          Data.send(code)
        end

        def find!(language_code)
           find(language_code) || raise(ArgumentError, 'Language "%s" is not found' % language_code)
        end
      end
    end

    class PaymentMethods
      class << self
        def find(payment_code)
          code = payment_code.gsub(/[^\w]/,'').capitalize.gsub(/_(\w)/){ $1.upcase }
          return nil unless Data.respond_to?(code)
          Data.send(code)
        end

        def find!(payment_code)
           find(payment_code) || raise(ArgumentError, 'Payment method "%s" is not found' % payment_code)
        end
      end
    end
  end
end

module Io::Flow::Reference::V0::Models
  class Currency
    def to_cents(amount_in)
      (amount_in * (10**number_decimals)).round(0)
    end
  end

  class Continent
  end

  class Country
    def locales
      list = 'country_%s' % @iso_3166_3.downcase
      ref  = ::FlowCommerce::Reference::Locales::Data
      ref.respond_to?(list) ? ref.send(list) : nil
    end
  end
end
